@@ -1,13 +1,18 @@
+# -*- coding: utf-8 -*-
+from __future__ import unicode_literals, division, print_function, absolute_import
 import sys
 import os
 import datetime
 import logging
 from contextlib import contextmanager
-import exceptions
+import uuid as uuidgen
 
 # first party
 from ..query import Query
 from ..exception import InterfaceError
+from ..decorators import reconnecting
+from ..compat import *
+
 
 logger = logging.getLogger(__name__)
 
@@ -17,7 +22,7 @@ class Connection(object):
 
     transaction_count = 0
     """
-    counting semaphore, greater than 0 if in a transaction, 0 if no current current transaction.
+    counting semaphore, greater than 0 if in a transaction, 0 if no current transaction.
 
     This will be incremented everytime transaction_start() is called, and decremented
     everytime transaction_stop() is called.
@@ -25,31 +30,43 @@ class Connection(object):
     transaction_fail will set this back to 0 and rollback the transaction
     """
 
+    def transaction_name(self):
+        """generate a random transaction name for use in start_transaction() and
+        fail_transaction()"""
+        name = uuidgen.uuid4()
+        return "p{}".format(str(name.hex))
+
     def in_transaction(self):
         """return true if currently in a transaction"""
         return self.transaction_count > 0
 
-    def transaction_start(self):
+    def transaction_start(self, name):
         """
         start a transaction
 
         this will increment transaction semaphore and pass it to _transaction_start()
         """
+        if not name:
+            raise ValueError("Transaction name cannot be empty")
+            #uid = id(self)
+
         self.transaction_count += 1
+        logger.debug("{}. Start transaction {}".format(self.transaction_count, name))
         if self.transaction_count == 1:
             self._transaction_start()
         else:
-            self._transaction_started()
+            self._transaction_started(name)
 
         return self.transaction_count
 
     def _transaction_start(self): pass
 
-    def _transaction_started(self): pass
+    def _transaction_started(self, name): pass
 
     def transaction_stop(self):
         """stop/commit a transaction if ready"""
         if self.transaction_count > 0:
+            logger.debug("{}. Stop transaction".format(self.transaction_count))
             if self.transaction_count == 1:
                 self._transaction_stop()
 
@@ -59,23 +76,31 @@ class Connection(object):
 
     def _transaction_stop(self): pass
 
-    def transaction_fail(self):
+    def transaction_fail(self, name):
         """
         rollback a transaction if currently in one
 
         e -- Exception() -- if passed in, bubble up the exception by re-raising it
         """
+        if not name:
+            raise ValueError("Transaction name cannot be empty")
+
         if self.transaction_count > 0:
+            logger.debug("{}. Failing transaction {}".format(self.transaction_count, name))
             if self.transaction_count == 1:
                 self._transaction_fail()
             else:
-                self._transaction_failing()
+                self._transaction_failing(name)
 
             self.transaction_count -= 1
 
     def _transaction_fail(self): pass
 
-    def _transaction_failing(self): pass
+    def _transaction_failing(self, name): pass
+
+#     def cursor(self, *args, **kwargs):
+#         pout.v("in transaction? {}".format(self.in_transaction()))
+#         return super(Connection, self).cursor(*args, **kwargs)
 
 
 class SQLConnection(Connection):
@@ -83,10 +108,10 @@ class SQLConnection(Connection):
         cur = self.cursor()
         cur.execute("BEGIN")
 
-    def _transaction_started(self):
+    def _transaction_started(self, name):
         cur = self.cursor()
         # http://www.postgresql.org/docs/9.2/static/sql-savepoint.html
-        cur.execute("SAVEPOINT prom{}".format(id(self)))
+        cur.execute("SAVEPOINT {}".format(name))
 
     def _transaction_stop(self):
         """
@@ -100,10 +125,10 @@ class SQLConnection(Connection):
         cur = self.cursor()
         cur.execute("ROLLBACK")
 
-    def _transaction_failing(self):
+    def _transaction_failing(self, name):
         cur = self.cursor()
         # http://www.postgresql.org/docs/9.2/static/sql-rollback-to.html
-        cur.execute("ROLLBACK TO SAVEPOINT prom{}".format(id(self)))
+        cur.execute("ROLLBACK TO SAVEPOINT {}".format(name))
 
 
 class Interface(object):
@@ -114,6 +139,14 @@ class Interface(object):
     connection_config = None
     """a config.Connection() instance"""
 
+    @classmethod
+    def configure(cls, connection_config):
+        host = connection_config.host
+        if host:
+            db = connection_config.database
+            connection_config.database = db.strip("/")
+        return connection_config
+
     def __init__(self, connection_config=None):
         self.connection_config = connection_config
 
@@ -130,13 +163,12 @@ class Interface(object):
 
         if connection_config: self.connection_config = connection_config
 
-        self.connected = False
-        self.transaction_count = 0
+        self.connected = True
         try:
             self._connect(self.connection_config)
-            self.connected = True
 
         except Exception as e:
+            self.connected = False
             self.raise_error(e)
 
         self.log("Connected {}", self.connection_config.interface_name)
@@ -148,6 +180,8 @@ class Interface(object):
 
     def get_connection(self): raise NotImplementedError()
 
+    def is_connected(self): return self.connected
+
     @contextmanager
     def connection(self, connection=None, **kwargs):
         try:
@@ -175,12 +209,24 @@ class Interface(object):
 
         self._close()
         self.connected = False
-        self.transaction_count = 0
         self.log("Closed Connection {}", self.connection_config.interface_name)
         return True
 
     def _close(self): raise NotImplementedError()
 
+    def readonly(self, readonly=True):
+        """Make the connection read only (pass in True) or read/write (pass in False)
+
+        :param readonly: boolean, True if this connection should be readonly, False
+            if the connection should be read/write
+        """
+        self.connection_config.readonly = readonly
+
+        if self.connected:
+            self._readonly(readonly)
+
+    def _readonly(self, readonly): raise NotImplementedError()
+
     def query(self, query_str, *query_args, **query_options):
         """
         run a raw query on the db
@@ -209,13 +255,14 @@ class Interface(object):
             # those db calls will be committed by this line
         """
         with self.connection(connection) as connection:
-            connection.transaction_start()
+            name = connection.transaction_name()
+            connection.transaction_start(name)
             try:
                 yield connection
                 connection.transaction_stop()
 
             except Exception as e:
-                connection.transaction_fail()
+                connection.transaction_fail(name)
                 self.raise_error(e)
 
     def set_table(self, schema, **kwargs):
@@ -226,15 +273,20 @@ class Interface(object):
         """
         with self.connection(**kwargs) as connection:
             kwargs['connection'] = connection
-            if self.has_table(schema.table, **kwargs): return True
+            if self.has_table(str(schema), **kwargs): return True
 
             try:
                 with self.transaction(**kwargs):
                     self._set_table(schema, **kwargs)
 
-                    for index_name, index_d in schema.indexes.iteritems():
-                        index_d['connection'] = connection
-                        self.set_index(schema, **index_d)
+                    for index_name, index in schema.indexes.items():
+                        self.set_index(
+                            schema,
+                            name=index.name,
+                            fields=index.fields,
+                            connection=connection,
+                            **index.options
+                        )
 
             except InterfaceError:
                 # check to see if this table now exists, it might have been created
@@ -265,10 +317,15 @@ class Interface(object):
         """
         with self.connection(**kwargs) as connection:
             kwargs['connection'] = connection
-            return self._get_tables(table_name, **kwargs)
+            return self._get_tables(str(table_name), **kwargs)
 
     def _get_tables(self, table_name, **kwargs): raise NotImplementedError()
 
+    def unsafe_delete_table(self, schema, **kwargs):
+        """wrapper around delete_table that matches the *_tables variant and denotes
+        that this is a serious operation"""
+        return self.delete_table(schema, **kwargs)
+
     def delete_table(self, schema, **kwargs):
         """
         remove a table matching schema from the db
@@ -277,7 +334,7 @@ class Interface(object):
         """
         with self.connection(**kwargs) as connection:
             kwargs['connection'] = connection
-            if not self.has_table(schema.table, **kwargs): return True
+            if not self.has_table(str(schema), **kwargs): return True
             with self.transaction(**kwargs):
                 self._delete_table(schema, **kwargs)
 
@@ -285,6 +342,13 @@ class Interface(object):
 
     def _delete_table(self, schema): raise NotImplementedError()
 
+    def unsafe_delete_tables(self):
+        """wrapper around delete_tables that is easier to remember but denotes the
+        same thing, I could never remember `disable_protection`
+
+        https://github.com/Jaymon/prom/issues/75"""
+        return self.delete_tables(disable_protection=True)
+
     def delete_tables(self, **kwargs):
         """
         removes all the tables from the db
@@ -302,6 +366,13 @@ class Interface(object):
 
     def _delete_tables(self, **kwargs): raise NotImplementedError()
 
+    def get_fields(self, table_name, **kwargs):
+        with self.connection(**kwargs) as connection:
+            kwargs['connection'] = connection
+            return self._get_fields(str(table_name), **kwargs)
+
+    def _get_fields(self, table_name, **kwargs): raise NotImplementedError()
+
     def get_indexes(self, schema, **kwargs):
         """
         get all the indexes
@@ -334,112 +405,67 @@ class Interface(object):
     def _set_index(self, schema, name, fields, **index_options):
         raise NotImplementedError()
 
-    def prepare_dict(self, schema, d, is_insert):
-        """
-        prepare the dict for insert/update
-
-        is_insert -- boolean -- True if insert, False if update
-        return -- dict -- the same dict, but now prepared
-        """
-        # TODO -- should this be moved to somewhere else? Not sure it is appropriate here
-        # update the times
-        now = datetime.datetime.utcnow()
-        field_created = schema._created
-        field_updated = schema._updated
-        if is_insert:
-            if field_created not in d:
-                d[field_created] = now
-
-        if field_updated not in d:
-            d[field_updated] = now
-
-        return d
-
-    def insert(self, schema, d, **kwargs):
+    @reconnecting()
+    def insert(self, schema, fields, **kwargs):
         """
         Persist d into the db
 
         schema -- Schema()
-        d -- dict -- the values to persist
+        fields -- dict -- the values to persist
 
-        return -- dict -- the dict that was inserted into the db
+        return -- int -- the primary key of the row just inserted
         """
-        d = self.prepare_dict(schema, d, is_insert=True)
         r = 0
 
         with self.connection(**kwargs) as connection:
             kwargs['connection'] = connection
             try:
                 with self.transaction(**kwargs):
-                    r = self._insert(schema, d, **kwargs)
+                    r = self._insert(schema, fields, **kwargs)
 
             except Exception as e:
                 exc_info = sys.exc_info()
                 if self.handle_error(schema, e, **kwargs):
-                    r = self._insert(schema, d, **kwargs)
+                    r = self._insert(schema, fields, **kwargs)
                 else:
                     self.raise_error(e, exc_info)
 
-        if r: d[schema._id] = r
-        return d
+        return r
 
-    def _insert(self, schema, d, **kwargs):
-        """return -- id -- the _id value"""
-        raise NotImplementedError()
+    def _insert(self, schema, fields, **kwargs): raise NotImplementedError()
 
-    def update(self, schema, query, **kwargs):
+    @reconnecting()
+    def update(self, schema, fields, query, **kwargs):
         """
         Persist the query.fields into the db that match query.fields_where
 
         schema -- Schema()
+        fields -- dict -- the values to persist
         query -- Query() -- will be used to create the where clause
 
-        return -- dict -- the dict that was inserted into the db
+        return -- int -- how many rows where updated
         """
-        d = query.fields
-        d = self.prepare_dict(schema, d, is_insert=False)
-
         with self.connection(**kwargs) as connection:
             kwargs['connection'] = connection
             try:
                 with self.transaction(**kwargs):
-                    r = self._update(schema, query, d, **kwargs)
+                    r = self._update(schema, fields, query, **kwargs)
 
             except Exception as e:
                 exc_info = sys.exc_info()
                 if self.handle_error(schema, e, **kwargs):
-                    r = self._update(schema, query, d, **kwargs)
+                    r = self._update(schema, fields, query, **kwargs)
                 else:
                     self.raise_error(e, exc_info)
 
-        return d
+        return r
 
-    def _update(self, schema, query, d, **kwargs): raise NotImplementedError()
-
-    def set(self, schema, query, **kwargs):
-        """
-        set d into the db, this is just a convenience method that will call either insert
-        or update depending on if query has a where clause
-
-        schema -- Schema()
-        query -- Query() -- set a where clause to perform an update, insert otherwise
-        return -- dict -- the dict inserted into the db
-        """
-        if query.fields_where:
-            d = self.update(schema, query, **kwargs)
-
-        else:
-            # insert
-            d = query.fields
-            d = self.insert(schema, d, **kwargs)
-
-        return d
+    def _update(self, schema, fields, query, **kwargs): raise NotImplementedError()
 
+    @reconnecting()
     def _get_query(self, callback, schema, query=None, *args, **kwargs):
-        """
-        this is just a common wrapper around all the get queries since they are
-        all really similar in how they execute
-        """
+        """this is just a common wrapper around all the get queries since they are
+        all really similar in how they execute"""
         if not query: query = Query()
 
         ret = None
@@ -464,7 +490,6 @@ class Interface(object):
                 else:
                     self.raise_error(e, exc_info)
 
-
         return ret
 
     def get_one(self, schema, query=None, **kwargs):
@@ -515,6 +540,13 @@ class Interface(object):
 
     def _delete(self, schema, query, **kwargs): raise NotImplementedError()
 
+    def spawn(self):
+        """Return a new instance of this Interface with the same connection configuration
+
+        :returns: Interface instance
+        """
+        return type(self)(self.connection_config)
+
     def handle_error(self, schema, e, **kwargs):
         """
         try and handle the error, return False if the error can't be handled
@@ -549,24 +581,32 @@ class Interface(object):
         """this is just a wrapper to make the passed in exception an InterfaceError"""
         if not exc_info:
             exc_info = sys.exc_info()
+
         if not isinstance(e, InterfaceError):
             # allow python's built in errors to filter up through
             # https://docs.python.org/2/library/exceptions.html
-            if not hasattr(exceptions, e.__class__.__name__):
-                e = InterfaceError(e, exc_info)
-        raise e.__class__, e, exc_info[2]
+            #if not hasattr(exceptions, e.__class__.__name__):
+            if not hasattr(builtins, e.__class__.__name__):
+                e = self._create_error(e, exc_info)
+
+        reraise(e.__class__, e, exc_info[2])
+
+    def _create_error(self, e, exc_info):
+        """create the error that you want to raise, this gives you an opportunity
+        to customize the error"""
+        return InterfaceError(e, exc_info)
 
 
 class SQLInterface(Interface):
     """Generic base class for all SQL derived interfaces"""
     @property
     def val_placeholder(self):
-        raise NotImplemented("this property should be set in any children class")
+        raise NotImplementedError("this property should be set in any children class")
 
     def _delete_tables(self, **kwargs):
-        for table_name in self.get_tables(**kwargs):
-            with self.transaction(**kwargs) as connection:
-                kwargs['connection'] = connection
+        with self.transaction(**kwargs) as connection:
+            kwargs['connection'] = connection
+            for table_name in self.get_tables(**kwargs):
                 self._delete_table(table_name, **kwargs)
 
         return True
@@ -591,32 +631,31 @@ class SQLInterface(Interface):
         ret = True
         # http://stackoverflow.com/questions/6739355/dictcursor-doesnt-seem-to-work-under-psycopg2
         connection = query_options.get('connection', None)
-        with self.connection(query_options.get('connection', None)) as connection:
+        with self.connection(connection) as connection:
             cur = connection.cursor()
             ignore_result = query_options.get('ignore_result', False)
             count_result = query_options.get('count_result', False)
-            one_result = query_options.get('fetchone', False)
-            cur_result = query_options.get('cursor_result', False)
+            one_result = query_options.get('fetchone', query_options.get('one_result', False))
+            cursor_result = query_options.get('cursor_result', False)
 
             try:
-                if not query_args:
+                if query_args:
+                    self.log("{}{}{}", query_str, os.linesep, query_args)
+                    cur.execute(query_str, query_args)
+                else:
                     self.log(query_str)
                     cur.execute(query_str)
 
-                else:
-                    self.log("{}{}{}", query_str, os.linesep, query_args)
-                    cur.execute(query_str, query_args)
-
-                if cur_result:
+                if cursor_result:
                     ret = cur
 
                 elif not ignore_result:
-                        if one_result:
-                            ret = cur.fetchone()
-                        elif count_result:
-                            ret = cur.rowcount
-                        else:
-                            ret = cur.fetchall()
+                    if one_result:
+                        ret = self._normalize_result_dict(cur.fetchone())
+                    elif count_result:
+                        ret = cur.rowcount
+                    else:
+                        ret = self._normalize_result_list(cur.fetchall())
 
             except Exception as e:
                 self.log(e)
@@ -624,66 +663,76 @@ class SQLInterface(Interface):
 
             return ret
 
-    def _normalize_date_SQL(self, field_name, field_kwargs):
+    def _normalize_result_dict(self, row):
+        return row
+
+    def _normalize_result_list(self, rows):
+        return rows
+
+    def _normalize_date_SQL(self, field_name, field_kwargs, symbol):
         raise NotImplemented()
 
-    def _normalize_field_SQL(self, schema, field_name):
-        return field_name, self.val_placeholder
-
-    def _normalize_list_SQL(self, schema, symbol_map, field_name, field_vals, field_kwargs=None):
-
-        format_str = ''
-        format_args = []
-        symbol = symbol_map['symbol']
-
-        if field_kwargs:
-            f = schema.fields[field_name]
-            if issubclass(f['type'], (datetime.datetime, datetime.date)):
-                format_strs = self._normalize_date_SQL(field_name, field_kwargs)
-                for fname, fvstr, fargs in format_strs:
-                    if format_str:
-                        format_str += ' AND '
-
-                    format_str += '{} {} ({})'.format(fname, symbol, ', '.join([fvstr] * len(fargs)))
-                    format_args.extend(fargs)
-
-            else:
-                raise ValueError('Field {} does not support extended kwarg values'.format(field_name))
-
-        else:
-            field_name, format_val_str = self._normalize_field_SQL(schema, field_name)
-            format_str = '{} {} ({})'.format(field_name, symbol, ', '.join([format_val_str] * len(field_vals)))
-            format_args.extend(field_vals)
-
-        return format_str, format_args
+    def _normalize_field_SQL(self, schema, field_name, symbol):
+        return self._normalize_name(field_name), self.val_placeholder
 
     def _normalize_val_SQL(self, schema, symbol_map, field_name, field_val, field_kwargs=None):
 
         format_str = ''
         format_args = []
+        symbol = symbol_map['symbol']
+        is_list = symbol_map.get('list', False)
 
         if field_kwargs:
-            symbol = symbol_map['symbol']
             # kwargs take precedence because None is a perfectly valid field_val
             f = schema.fields[field_name]
-            if issubclass(f['type'], (datetime.datetime, datetime.date)):
-                format_strs = self._normalize_date_SQL(field_name, field_kwargs)
+            if issubclass(f.type, (datetime.datetime, datetime.date)):
+                format_strs = self._normalize_date_SQL(field_name, field_kwargs, symbol)
                 for fname, fvstr, farg in format_strs:
                     if format_str:
                         format_str += ' AND '
 
-                    format_str += '{} {} {}'.format(fname, symbol, fvstr)
-                    format_args.append(farg)
+
+                    if is_list:
+                        format_str += '{} {} ({})'.format(
+                            fname,
+                            symbol,
+                            ', '.join([fvstr] * len(farg))
+                        )
+                        format_args.extend(farg)
+
+                    else:
+                        format_str += '{} {} {}'.format(fname, symbol, fvstr)
+                        format_args.append(farg)
 
             else:
                 raise ValueError('Field {} does not support extended kwarg values'.format(field_name))
 
         else:
-            # special handling for NULL
-            symbol = symbol_map['none_symbol'] if field_val is None else symbol_map['symbol']
-            field_name, format_val_str = self._normalize_field_SQL(schema, field_name)
-            format_str = '{} {} {}'.format(field_name, symbol, format_val_str)
-            format_args.append(field_val)
+            if is_list:
+                field_name, format_val_str = self._normalize_field_SQL(schema, field_name, symbol)
+                format_str = '{} {} ({})'.format(
+                    field_name,
+                    symbol,
+                    ', '.join([format_val_str] * len(field_val))
+                )
+                format_args.extend(field_val)
+
+            else:
+                # special handling for NULL
+                if field_val is None:
+                    symbol = symbol_map['none_symbol']
+
+                field_name, format_val_str = self._normalize_field_SQL(
+                    schema,
+                    field_name,
+                    symbol
+                )
+                format_str = '{} {} {}'.format(
+                    field_name,
+                    symbol,
+                    format_val_str
+                )
+                format_args.append(field_val)
 
         return format_str, format_args
 
@@ -693,6 +742,19 @@ class SQLInterface(Interface):
         return -- tuple -- field_sort_str, field_sort_args"""
         raise NotImplemented()
 
+    def _normalize_table_name(self, schema):
+        return self._normalize_name(schema)
+
+    def _normalize_name(self, name):
+        """normalize a non value name for the query
+
+        https://blog.christosoft.de/2012/10/sqlite-escaping-table-acolumn-names/
+
+        :param name: str, the name that should be prepared to be queried
+        :returns: the modified name ready to be added to a query string
+        """
+        return '"{}"'.format(name)
+
     def get_SQL(self, schema, query, **sql_options):
         """
         convert the query instance into SQL
@@ -706,14 +768,18 @@ class SQLInterface(Interface):
         """
         only_where_clause = sql_options.get('only_where_clause', False)
         symbol_map = {
-            'in': {'args': self._normalize_list_SQL, 'symbol': 'IN'},
-            'nin': {'args': self._normalize_list_SQL, 'symbol': 'NOT IN'},
-            'is': {'arg': self._normalize_val_SQL, 'symbol': '=', 'none_symbol': 'IS'},
-            'not': {'arg': self._normalize_val_SQL, 'symbol': '!=', 'none_symbol': 'IS NOT'},
-            'gt': {'arg': self._normalize_val_SQL, 'symbol': '>'},
-            'gte': {'arg': self._normalize_val_SQL, 'symbol': '>='},
-            'lt': {'arg': self._normalize_val_SQL, 'symbol': '<'},
-            'lte': {'arg': self._normalize_val_SQL, 'symbol': '<='},
+            'in': {'symbol': 'IN', 'list': True},
+            'nin': {'symbol': 'NOT IN', 'list': True},
+            'is': {'symbol': '=', 'none_symbol': 'IS'},
+            'not': {'symbol': '!=', 'none_symbol': 'IS NOT'},
+            'gt': {'symbol': '>'},
+            'gte': {'symbol': '>='},
+            'lt': {'symbol': '<'},
+            'lte': {'symbol': '<='},
+            # https://www.tutorialspoint.com/postgresql/postgresql_like_clause.htm
+            # https://www.tutorialspoint.com/sqlite/sqlite_like_clause.htm
+            'like': {'symbol': 'LIKE'},
+            'nlike': {'symbol': 'NOT LIKE'},
         }
 
         query_args = []
@@ -721,18 +787,23 @@ class SQLInterface(Interface):
 
         if not only_where_clause:
             query_str.append('SELECT')
+            select_fields = query.fields_select
+            if select_fields:
+                distinct = "DISTINCT " if select_fields.options.get("unique", False) else ""
+                select_fields_str = distinct + ',{}'.format(os.linesep).join(
+                    (self._normalize_name(f) for f in select_fields.names())
+                )
+            else:
+                select_fields_str = "*"
 
             if sql_options.get('count_query', False):
-                query_str.append('  count(*) as ct')
+                query_str.append('  count({}) as ct'.format(select_fields_str))
+
             else:
-                select_fields = query.fields_select
-                if select_fields:
-                    query_str.append('  ' + ',{}'.format(os.linesep).join(select_fields))
-                else:
-                    query_str.append('  *')
+                query_str.append('  {}'.format(select_fields_str))
 
             query_str.append('FROM')
-            query_str.append('  {}'.format(schema))
+            query_str.append("  {}".format(self._normalize_table_name(schema)))
 
         if query.fields_where:
             query_str.append('WHERE')
@@ -746,12 +817,13 @@ class SQLInterface(Interface):
 
                 # field[0], field[1], field[2], field[3]
                 _, field_name, field_val, field_kwargs = field
-
-                if 'args' in sd:
-                    field_str, field_args = sd['args'](schema, sd, field_name, field_val, field_kwargs)
-
-                elif 'arg' in sd:
-                    field_str, field_args = sd['arg'](schema, sd, field_name, field_val, field_kwargs)
+                field_str, field_args = self._normalize_val_SQL(
+                    schema,
+                    sd,
+                    field_name,
+                    field_val,
+                    field_kwargs
+                )
 
                 query_str.append('  {}'.format(field_str))
                 query_args.extend(field_args)
@@ -772,9 +844,12 @@ class SQLInterface(Interface):
             query_str.append(',{}'.format(os.linesep).join(query_sort_str))
 
         if query.bounds:
-            limit, offset, _ = query.get_bounds()
-            if limit > 0:
-                query_str.append('LIMIT {} OFFSET {}'.format(limit, offset))
+            offset = query.bounds.offset
+            limit = 1 if sql_options.get('one_query', False) else query.bounds.limit
+            query_str.append('LIMIT {} OFFSET {}'.format(
+                limit,
+                offset
+            ))
 
         query_str = os.linesep.join(query_str)
         return query_str, query_args
@@ -784,8 +859,8 @@ class SQLInterface(Interface):
         if not connection: return False
 
         ret = False
-        if connection.closed == 0:
-            connection.transaction_stop()
+        if connection.closed == 0: # connection is open
+            #connection.transaction_stop()
             if isinstance(e, InterfaceError):
                 ret = self._handle_error(schema, e.e, **kwargs)
 
@@ -812,8 +887,8 @@ class SQLInterface(Interface):
         with self.transaction(**kwargs) as connection:
             kwargs['connection'] = connection
             # go through and make sure all foreign key referenced tables exist
-            for field_name, field_val in schema.fields.iteritems():
-                s = field_val.ref_schema
+            for field_name, field_val in schema.fields.items():
+                s = field_val.schema
                 if s:
                     self._set_all_tables(s, **kwargs)
 
@@ -822,32 +897,27 @@ class SQLInterface(Interface):
 
         return True
 
-    def _update(self, schema, query, d, **kwargs):
+    def _update(self, schema, fields, query, **kwargs):
         where_query_str, where_query_args = self.get_SQL(schema, query, only_where_clause=True)
-        pk_name = schema.pk
-
         query_str = 'UPDATE {} SET {} {}'
         query_args = []
 
         field_str = []
-        for field_name, field_val in d.iteritems():
-            field_str.append('{} = {}'.format(field_name, self.val_placeholder))
+        for field_name, field_val in fields.items():
+            field_str.append('{} = {}'.format(self._normalize_name(field_name), self.val_placeholder))
             query_args.append(field_val)
 
         query_str = query_str.format(
-            schema.table,
+            self._normalize_table_name(schema),
             ',{}'.format(os.linesep).join(field_str),
             where_query_str
         )
         query_args.extend(where_query_args)
 
-        return self.query(query_str, *query_args, ignore_result=True, **kwargs)
+        return self.query(query_str, *query_args, count_result=True, **kwargs)
 
     def _get_one(self, schema, query, **kwargs):
-        # compensate for getting one with an offset
-        if query.has_bounds() and not query.has_limit():
-            query.set_limit(1)
-        query_str, query_args = self.get_SQL(schema, query)
+        query_str, query_args = self.get_SQL(schema, query, one_query=True)
         return self.query(query_str, *query_args, fetchone=True, **kwargs)
 
     def _get(self, schema, query, **kwargs):
@@ -870,10 +940,10 @@ class SQLInterface(Interface):
         the reason they have to be NULL is adding fields to Postgres that can be NULL
         is really light, but if they have a default value, then it can be costly
         """
-        current_fields = self._get_fields(schema, **kwargs)
-        for field_name, field_options in schema.fields.iteritems():
+        current_fields = self.get_fields(schema, **kwargs)
+        for field_name, field in schema.fields.items():
             if field_name not in current_fields:
-                if field_options.get('required', False):
+                if field.required:
                     raise ValueError('Cannot safely add {} on the fly because it is required'.format(field_name))
 
                 else:
@@ -881,7 +951,7 @@ class SQLInterface(Interface):
                     query_str.append('ALTER TABLE')
                     query_str.append('  {}'.format(schema))
                     query_str.append('ADD COLUMN')
-                    query_str.append('  {}'.format(self.get_field_SQL(field_name, field_options)))
+                    query_str.append('  {}'.format(self.get_field_SQL(field_name, field)))
                     query_str = os.linesep.join(query_str)
                     self.query(query_str, ignore_result=True, **kwargs)
 
